from datetime import datetime
from math import floor, log10
from opa_tools.config import get_request_dir
from typing import Union
import yaml

from .retrieval import (
    get_request,
    _normalize_request_identifier,
    _normalize_request_month,
    _find_zero_padded_subdir,
)
from .transformation import product, unflatten, wrap


REQUEST_DIR = get_request_dir()


def _write_requests(requests, grequest_dir):
    with open(grequest_dir / "date.txt", "w") as wf:
        wf.write(datetime.now().isoformat())
    n_digits = floor(log10(len(requests))) + 1
    for i, request in enumerate(requests, start=1):
        request_dir = REQUEST_DIR / grequest_dir / f"{i:0{n_digits}d}"
        request_dir.mkdir()
        gsvrequest, oparequest = unflatten(request)
        with (
            open(request_dir / "gsv.yml", "w") as gsv_f,
            open(request_dir / "opa.yml", "w") as opa_f,
        ):
            yaml.dump(gsvrequest, gsv_f)
            yaml.dump(oparequest, opa_f)


def create_template(
    src: str,
    dst: str,
) -> None:
    src_request = get_request(src, flat=False, single_as_list=False)
    new_dir = REQUEST_DIR / _normalize_request_month(None, None) / dst
    if new_dir.exists():
        raise IsADirectoryError(
            f"Directory '{new_dir}' exists. Choose a different dst."
        )
    new_dir.mkdir()
    new_path = new_dir / "grequest.yaml"
    dummy_wrap = wrap([src_request])
    with open(new_path, "w") as wf:
        yaml.dump(dummy_wrap, wf)
    print(f"File {new_path} created.")


def generate_requests(
    identifier: Union[str, int],
) -> None:
    name, _, month, _ = _normalize_request_identifier(identifier, None, None)
    grequest_path = (
        _find_zero_padded_subdir(REQUEST_DIR / month, name) / "grequest.yaml"
    )
    if not grequest_path.exists():
        raise FileNotFoundError(
            f"Generating request attempted to be found in '{grequest_path}', but none found. "
            "Have you run `create_template`?"
        )
    with open(grequest_path, "r") as rf:
        generating_request = yaml.safe_load(rf)
    requests = product(generating_request)
    _write_requests(requests, grequest_path.parent)
