from collections import defaultdict
import itertools
from typing import Dict, Sequence, Tuple
import warnings

from opa_tools.utils import process_dct


def flatten(gsvrequest, oparequest, type_changed_keys=None):
    return {
        **process_dct(gsvrequest, prefix="gsv_", type_changed_keys=type_changed_keys),
        **process_dct(oparequest, prefix="opa_", type_changed_keys=type_changed_keys),
    }


def unflatten(flatrequest):
    return (
        {
            key.removeprefix("gsv_"): value
            for key, value in flatrequest.items()
            if key.startswith("gsv_")
        },
        {
            key.removeprefix("opa_"): value
            for key, value in flatrequest.items()
            if key.startswith("opa_")
        },
    )


def product(generating_request):
    requests = []
    generating_request_items = list(generating_request.items())
    req_keys, req_values = zip(*generating_request_items)
    for value_comb in itertools.product(*req_values):
        requests.append(dict(zip(req_keys, value_comb)))
    return requests


def wrap(requests: Sequence[Tuple[Dict, Dict]]):
    req_dcts = defaultdict(set)
    type_changed_keys = set()
    for gsvrequest, oparequest in requests:
        request = flatten(gsvrequest, oparequest, type_changed_keys)
        for key, value in request.items():
            req_dcts[key].add(value)
    req_dcts = {key: list(values) for key, values in req_dcts.items()}
    if type_changed_keys:
        warnings.warn(
            "Please, review and correct the types for the "
            "following keys in the wrapped request:"
            "\n- " + "\n- ".join(sorted(type_changed_keys))
        )
    return req_dcts
