from datetime import datetime
from typing import Sequence, Tuple


def _add_date(datetimes, date):
    if date == "0":
        datetimes.append(datetime(1900, 1, 1))
    elif date == "E":
        datetimes.append(datetime(datetime.now().year + 1, 1, 1))
    else:
        datetimes.append(datetime.fromisoformat(date))

def add_date_parsing_args(parser):
    parser.add_argument(
        "-d",
        "--dates",
        action="store",
        type=str,
        nargs="*",
        metavar="DATES",
    )
    parser.add_argument(
        "-i",
        "--interval",
        action="store_true",
        help="Wether the passed dates are an interval. Then, they must be two and in isoformat.",
    )

def parse_date_interval(dates: Sequence[str]) -> Tuple[datetime, datetime]:
    if len(dates) != 2:
        raise ValueError(
            "When passing an interval, there should only be two dates (min and max)."
        )
    datetimes = []
    try:
        for date in dates:
            _add_date(datetimes, date)
    except ValueError as exc:
        raise ValueError(
            "When passing an interval, dates should be passed in isoformat."
        ) from exc
    return tuple(datetimes)

def process_dct(
    dct, prefix="", valid_types=(str, int, float), force_fun=str, wrapper_fun=None, *, type_changed_keys=None
):
    if dct is None:
        return {}
    processed_dict = {}
    for key, value in dct.items():
        processed_key = f"{prefix}{key}"
        if valid_types and force_fun and not isinstance(value, valid_types):
            processed_value = force_fun(value)
            if isinstance(type_changed_keys, set):
                type_changed_keys.add(processed_key)
        else:
            processed_value = value
        if wrapper_fun:
            processed_value = wrapper_fun(processed_value)
        processed_dict[processed_key] = processed_value
    return processed_dict

