
import ast
from collections import defaultdict
import numpy as np
from pathlib import Path
from typing import Optional, Sequence

import seaborn as sns
from matplotlib.pyplot import cm
import matplotlib.pyplot as plt
import matplotlib.patches as mpatches

from opa_tools.benchmarking.retrieval import get_benchmark_iterator, get_request_df
from opa_tools.benchmarking.processor import get_prof_time


sns.set_theme()

def _get_benchmark_date(benchmark_dir):
    with open(benchmark_dir / "date.txt", "r") as rf:
        return rf.read()

def _get_color_iterator(cm_name=None, n=10):
    return iter(cm.viridis(np.linspace(0, 1, n)))
    # return cm.get_cmap(cm_name)

def _include_legend(ax, colors):
    fig_legend = []
    for opa_stat, color in colors.items():
        fig_legend.append(
            mpatches.Patch(color=color, label=opa_stat)
        )
    ax.legend(handles=fig_legend, loc="best")

def plot_world_coverage(*dates: Sequence[str], exclude: Optional[Sequence[str]] = None, **kwargs):
    colors = defaultdict(str)
    request_df = get_request_df(*dates)
    color_iter = _get_color_iterator(n=len(set(request_df["opa_stat"]))
)
    fig, ax = plt.subplots(figsize=kwargs.get("figsize"))
    print(request_df)
    for benchmark_dir in get_benchmark_iterator(dates=dates, exclude=exclude):
        benchmark_date = _get_benchmark_date(benchmark_dir)
        benchmark_name = benchmark_dir.name
        for file in benchmark_dir.iterdir():
            benchmark_req = request_df[
                (request_df["date"] == benchmark_date) &
                (request_df["bname"] == benchmark_name)
            ]
            opa_stat = benchmark_req["opa_stat"].iloc[0]
            gsv_area = ast.literal_eval(benchmark_req["gsv_area"].iloc[0])
            world_coverage = (
                (gsv_area[0] - gsv_area[2]) * (gsv_area[3] - gsv_area[1])
            ) / (180 * 360)
            if file.suffix == ".prof":
                # TODO: Remove hardcoded restrictions
                time = get_prof_time(file, "one_pass", "compute")
                if opa_stat not in colors:
                    colors[opa_stat] = next(color_iter)
                color = colors[opa_stat]
                ax.plot([world_coverage], [time], "o", color=color, lw=0)
                ax.text(time, 0.5, benchmark_name)
                ax.text(time, -0.5, opa_stat)
                print(time)
    _include_legend(ax, colors)
    return fig


def main():
    fig = plot_world_coverage(
        "20241004",
        "20241007",
    )
    fig.savefig(Path.home() / "Pictures/world_coverage.png")


if __name__ == "__main__":
    main()
